/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package Nelder_Mead_algoritmus;

import Funkce.Funkce;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Random;

/**
 *
 * @author Miraxp
 */
public class Simplex {

    private List<Vrchol> vrcholy;
    private short pocetVrcholu, pocetHodnotVrcholu;
    private Vrchol teziste_x0, bod_xr, bod_xe, bod_xc;
    private double alfa, beta, gama, delta;
    private static Comparator<Vrchol> pComparator;
    private Funkce funkce;

    public Simplex(short nPocetHodnotVrch, final Funkce nova_funkce) {
        this.alfa = 1;
        this.beta = 2;
        this.gama = 0.5;
        this.delta = 0.5;
        this.pocetHodnotVrcholu = nPocetHodnotVrch;
        this.pocetVrcholu = (short) (pocetHodnotVrcholu + 1);
        vrcholy = new ArrayList(pocetVrcholu);
        generujSimplex();

        funkce = nova_funkce;

        pComparator = new Comparator<Vrchol>() {

            public int compare(Vrchol vrchol_1, Vrchol vrchol_2) {
                return Double.compare(funkce.f(vrchol_1.dejHodnoty()), funkce.f(vrchol_2.dejHodnoty()));
            }
        };
    }

    private void generujSimplex() {
        double noveHodnoty[] = new double[pocetHodnotVrcholu];
        Random nahodneCislo = new Random();
        for (int aktVrchol = 0; aktVrchol < pocetVrcholu; aktVrchol++) {
            for (int aktHodnota = 0; aktHodnota < pocetHodnotVrcholu; aktHodnota++) {
                noveHodnoty[aktHodnota] = nahodneCislo.nextDouble();
            }
            vrcholy.add(new Vrchol(noveHodnoty));
        }
    }

    public double[] dejHodnotyVrcholu(int indexVrcholu) {
        if (indexVrcholu < pocetVrcholu && indexVrcholu >= 0) {
            return vrcholy.get(indexVrcholu).dejHodnoty();
        } else {
            return null;
        }
    }

    void seradVrcholy() {
        Collections.sort(vrcholy, pComparator);
    }

    public void najdiTeziste() {
        double sumyHodnot[] = new double[pocetHodnotVrcholu];
        double hodnotyTeziste[] = new double[pocetHodnotVrcholu];
        double hodnotyVrcholu[];
        for (int aktVrchol = 0; aktVrchol < (pocetVrcholu - 1); aktVrchol++) {
            hodnotyVrcholu = dejHodnotyVrcholu(aktVrchol);
            for (int aktHodnota = 0; aktHodnota < pocetHodnotVrcholu; aktHodnota++) {
                if (aktVrchol == 0) {
                    sumyHodnot[aktHodnota] = 0;
                }
                sumyHodnot[aktHodnota] += hodnotyVrcholu[aktHodnota];
            }
        }
        for (int aktHodnota = 0; aktHodnota < pocetHodnotVrcholu; aktHodnota++) {
            hodnotyTeziste[aktHodnota] = sumyHodnot[aktHodnota] / (double) (pocetVrcholu - 1);
        }
        teziste_x0 = new Vrchol(hodnotyTeziste);
    }

    private void najdiBodXR() {
        double hodnotyBoduXR[] = new double[pocetHodnotVrcholu];
        double hodnotyTeziste[] = teziste_x0.dejHodnoty();
        double hodnotyNejhorsiho[] = vrcholy.get(pocetVrcholu - 1).dejHodnoty();
        for (int aktHodnota = 0; aktHodnota < pocetHodnotVrcholu; aktHodnota++) {
            hodnotyBoduXR[aktHodnota] = hodnotyTeziste[aktHodnota] + alfa * (hodnotyTeziste[aktHodnota] - hodnotyNejhorsiho[aktHodnota]);
        }
        bod_xr = new Vrchol(hodnotyBoduXR);
    }

    public double[] dejHodnotyBoduXR() {
        najdiBodXR();
        return bod_xr.dejHodnoty();
    }

    private void najdiBodXE() {
        double hodnotyBoduXE[] = new double[pocetHodnotVrcholu];
        double hodnotyTeziste[] = teziste_x0.dejHodnoty();
        double hodnotyNejhorsiho[] = vrcholy.get(pocetVrcholu - 1).dejHodnoty();
        for (int aktHodnota = 0; aktHodnota < pocetHodnotVrcholu; aktHodnota++) {
            hodnotyBoduXE[aktHodnota] = hodnotyTeziste[aktHodnota] + beta * (hodnotyTeziste[aktHodnota] - hodnotyNejhorsiho[aktHodnota]);
        }
        bod_xe = new Vrchol(hodnotyBoduXE);
    }

    public double[] dejHodnotyBoduXE() {
        najdiBodXE();
        return bod_xe.dejHodnoty();
    }

    private void najdiBodXC() {
        double hodnotyBoduXC[] = new double[pocetHodnotVrcholu];
        double hodnotyTeziste[] = teziste_x0.dejHodnoty();
        double hodnotyNejhorsiho[] = vrcholy.get(pocetVrcholu - 1).dejHodnoty();
        for (int aktHodnota = 0; aktHodnota < pocetHodnotVrcholu; aktHodnota++) {
            hodnotyBoduXC[aktHodnota] = hodnotyNejhorsiho[aktHodnota] + gama * (hodnotyTeziste[aktHodnota] - hodnotyNejhorsiho[aktHodnota]);
        }
        bod_xc = new Vrchol(hodnotyBoduXC);
    }

    public double[] dejHodnotyBoduXC() {
        najdiBodXC();
        return bod_xc.dejHodnoty();
    }

    public double[] dejHodnotyBodu_xi(int aktVrchol) {
        double hodnotyBoduXI[] = new double[pocetHodnotVrcholu];
        double hodnotyNejlepsiho[] = vrcholy.get(0).dejHodnoty();
        double hodnotyAktualniho[] = vrcholy.get(aktVrchol).dejHodnoty();
        for (int aktHodnota = 0; aktHodnota < pocetHodnotVrcholu; aktHodnota++) {
            hodnotyBoduXI[aktHodnota] = hodnotyNejlepsiho[aktHodnota] + delta * (hodnotyAktualniho[aktHodnota] - hodnotyNejlepsiho[aktHodnota]);
        }
        return hodnotyBoduXI;
    }

    // Reflection
    public void reflexe() {
        vrcholy.set(pocetVrcholu - 1, bod_xr);
    }

    // Expansion
    public void expanze() {
        vrcholy.set(pocetVrcholu - 1, bod_xe);
    }

    // Contraction
    public void kontrakce() {
        vrcholy.set(pocetVrcholu - 1, bod_xc);
    }
    // Reduction

    public void redukce() {
        for (int aktVrchol = 1; aktVrchol < pocetVrcholu; aktVrchol++) {
            vrcholy.set(aktVrchol, new Vrchol(dejHodnotyBodu_xi(aktVrchol)));
        }
    }

    public void vypis() {
        double hodnoty[] = null;
        for (int aktVrchol = 0; aktVrchol < pocetVrcholu; aktVrchol++) {
            hodnoty = dejHodnotyVrcholu(aktVrchol);
            for (int aktHodnota = 0; aktHodnota < pocetHodnotVrcholu; aktHodnota++) {
                System.out.print(hodnoty[aktHodnota] + " ");
            }
            System.out.println("");
        }
    }

    public boolean jsouSiRovny() {
        for (int aktVrchol = 0; aktVrchol < pocetVrcholu - 1; aktVrchol++) {
            if (Double.compare(funkce.f(vrcholy.get(aktVrchol).dejHodnoty()), funkce.f(vrcholy.get(aktVrchol + 1).dejHodnoty())) != 0) {
                return false;
            }
        }
        return true;
    }
}
